@@ -3,11 +3,10 @@
 # Copyright (c) 2019 Red Hat, Inc.
 # Copyright (c) 2019 Tomáš Mráz <tmraz@fedoraproject.org>
 
-from subprocess import check_output, CalledProcessError
+from subprocess import CalledProcessError, check_output
 
 from .configgenerator import ConfigGenerator
 
-
 RH_ALLOW_SHA1 = '''
 [openssl_init]
 alg_section = evp_properties
@@ -25,7 +24,6 @@ activate = 1
 
 class OpenSSLGenerator(ConfigGenerator):
     CONFIG_NAME = 'openssl'
-    SCOPES = {'tls', 'ssl', 'openssl'}
 
     cipher_not_map = {
         'AES-256-CTR': '',
@@ -134,10 +132,8 @@ class OpenSSLGenerator(ConfigGenerator):
         # These ciphers are not necessary for any
         # policy level, and only increase the attack surface.
         # FIXME! must be fixed for custom policies
-        s = cls.append(s, '-SHA384')
-        s = cls.append(s, '-CAMELLIA')
-        s = cls.append(s, '-ARIA')
-        s = cls.append(s, '-AESCCM8')
+        for c in ('-SHA384', '-CAMELLIA', '-ARIA', '-AESCCM8'):
+            s = cls.append(s, c)
 
         return s
 
@@ -154,15 +150,17 @@ class OpenSSLGenerator(ConfigGenerator):
         return s
 
     @classmethod
-    def generate_config(cls, policy):
+    def generate_config(cls, unscoped_policy):
+        policy = unscoped_policy.scoped({'tls', 'ssl', 'openssl'})
         return cls.generate_ciphers(policy) + '\n'
 
     @classmethod
     def test_config(cls, config):
         output = b''
-        assert config.endswith('\n')
+        assert config.endswith('\n')  # noqa: S101
         try:
-            output = check_output(['openssl', 'ciphers', config[:-1]])
+            output = check_output(['openssl',  # noqa: S607
+                                   'ciphers', config[:-1]])
         except CalledProcessError:
             cls.eprint('There is an error in openssl generated policy')
             cls.eprint(f'Policy:\n{config}')
@@ -216,6 +214,48 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
         'RSA-PSS-RSAE-SHA2-512': 'rsa_pss_rsae_sha512',
         'EDDSA-ED25519': 'ed25519',
         'EDDSA-ED448': 'ed448',
+        # provider-only, so, optional (openssl#23050) + marked experimental
+        'MLDSA44': '?mldsa44',
+        'P256-MLDSA44': '?p256_mldsa44',
+        'RSA3072-MLDSA44': '?rsa3072_mldsa44',
+        'MLDSA44-PSS2048': '?mldsa44_pss2048',
+        'MLDSA44-RSA2048': '?mldsa44_rsa2048',
+        'MLDSA44-ED25519': '?mldsa44_ed25519',
+        'MLDSA44-P256': '?mldsa44_p256',
+        'MLDSA44-BP256': '?mldsa44_bp256',
+        'MLDSA65': '?mldsa65',
+        'P384-MLDSA65': '?p384_mldsa65',
+        'MLDSA65-PSS3072': '?mldsa65_pss3072',
+        'MLDSA65-RSA3072': '?mldsa65_rsa3072',
+        'MLDSA65-P256': '?mldsa65_p256',
+        'MLDSA65-BP256': '?mldsa65_bp256',
+        'MLDSA65-ED25519': '?mldsa65_ed25519',
+        'MLDSA87': '?mldsa87',
+        'P521-MLDSA87': '?p521_mldsa87',
+        'MLDSA87-P384': '?mldsa87_p384',
+        'MLDSA87-BP384': '?mldsa87_bp384',
+        'MLDSA87-ED448': '?mldsa87_ed448',
+        'FALCON512': '?falcon512',
+        'P256-FALCON512': '?p256_falcon512',
+        'RSA3072-FALCON512': '?rsa3072_falcon512',
+        'FALCONPADDED512': '?falconpadded512',
+        'P256-FALCONPADDED512': '?p256_falconpadded512',
+        'RSA3072-FALCONPADDED512': '?rsa3072_falconpadded512',
+        'FALCON1024': '?falcon1024',
+        'P521-FALCON1024': '?p521_falcon1024',
+        'FALCONPADDED1024': '?falconpadded1024',
+        'P521-FALCONPADDED1024': '?p521_falconpadded1024',
+        'SPHINCSSHA2128FSIMPLE': '?sphincssha2128fsimple',
+        'P256-SPHINCSSHA2128FSIMPLE': '?p256_sphincssha2128fsimple',
+        'RSA3072-SPHINCSSHA2128FSIMPLE': '?rsa3072_sphincssha2128fsimple',
+        'SPHINCSSHA2128SSIMPLE': '?sphincssha2128ssimple',
+        'P256-SPHINCSSHA2128SSIMPLE': '?p256_sphincssha2128ssimple',
+        'RSA3072-SPHINCSSHA2128SSIMPLE': '?rsa3072_sphincssha2128ssimple',
+        'SPHINCSSHA2192FSIMPLE': '?sphincssha2192fsimple',
+        'P384-SPHINCSSHA2192FSIMPLE': '?p384_sphincssha2192fsimple',
+        'SPHINCSSHAKE128FSIMPLE': '?sphincsshake128fsimple',
+        'P256-SPHINCSSHAKE128FSIMPLE': '?p256_sphincsshake128fsimple',
+        'RSA3072-SPHINCSSHAKE128FSIMPLE': '?rsa3072_sphincsshake128fsimple',
     }
 
     group_map = {
@@ -233,10 +273,26 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
         'BRAINPOOL-P256R1': 'brainpoolP256r1',
         'BRAINPOOL-P384R1': 'brainpoolP384r1',
         'BRAINPOOL-P512R1': 'brainpoolP512r1',
+        # provider-only, so, optional (openssl#23050) + marked experimental
+        'KYBER768': '?kyber768',
+        'X25519-KYBER768': '?x25519_kyber768',
+        'P256-KYBER768': '?p256_kyber768',
+        'MLKEM512': '?mlkem512',
+        'P256-MLKEM512': '?p256_mlkem512',
+        'X25519-MLKEM512': '?x25519_mlkem512',
+        'MLKEM768': '?mlkem768',
+        'P384-MLKEM768': '?p384_mlkem768',
+        'X448-MLKEM768': '?x448_mlkem768',
+        'X25519-MLKEM768': '?X25519MLKEM768:?x25519_mlkem768',  # old/new name
+        'P256-MLKEM768': '?SecP256r1MLKEM768:?p256_mlkem768',  # old/new name
+        'MLKEM1024': '?mlkem1024',
+        'P521-MLKEM1024': '?p521_mlkem1024',
+        'P384-MLKEM1024': '?p384_mlkem1024',
     }
 
     @classmethod
-    def generate_config(cls, policy):
+    def generate_config(cls, unscoped_policy):
+        policy = unscoped_policy.scoped({'tls', 'ssl', 'openssl'})
         p = policy.enabled
         # This includes the seclevel
         s = f'CipherString = {cls.generate_ciphers(policy)}\n'
@@ -261,12 +317,12 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
         groups = [cls.group_map[i] for i in p['group'] if i in cls.group_map]
         s += 'Groups = ' + ':'.join(groups) + '\n'
 
-        if 'SHA1' in p['hash']:
-            s += RH_ALLOW_SHA1
-
         if policy.enums['__ems'] == 'RELAX':
             s += 'Options = RHNoEnforceEMSinFIPS\n'
 
+        if 'SHA1' in p['hash']:
+            s += RH_ALLOW_SHA1
+
         return s
 
     @classmethod
@@ -276,10 +332,10 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
 
 class OpenSSLFIPSGenerator(ConfigGenerator):
     CONFIG_NAME = 'openssl_fips'
-    SCOPES = {'tls', 'ssl', 'openssl'}
 
     @classmethod
-    def generate_config(cls, policy):
+    def generate_config(cls, unscoped_policy):
+        policy = unscoped_policy.scoped({'tls', 'ssl', 'openssl'})
         # OpenSSL EMS relaxation is special
         # in that it uses a separate FIPS module config
         # and, just in case, EMS is enforcing by default.
